--
-- PLB VGA component
-- Copyright (C) Jack Whitham 2009
-- http://www.jwhitham.org.uk/
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity user_logic is
  port
  (
        clk                 : in std_logic;
        rst                 : in std_logic;
        -- User interface for memory
        mem_user_address    : out std_logic_vector ( 31 downto 0 );
        mem_data_in         : in std_logic_vector ( 31 downto 0 );
        mem_data_out        : out std_logic_vector ( 31 downto 0 );
        mem_read            : out std_logic;
        mem_write           : out std_logic;
        mem_busy            : in std_logic;
        mem_byte_enable     : out std_logic_vector ( 3 downto 0 );

                -- User UART channel (in/out)
        uart_data_in        : in std_logic_vector ( 7 downto 0 );
        uart_data_out       : out std_logic_vector ( 7 downto 0 );
        uart_read           : out std_logic;
        uart_write          : out std_logic;
        uart_data_present   : in std_logic;
        uart_busy           : in std_logic;

                -- User connections for simpler hardware
        btn_north_in        : in std_logic;
        btn_east_in         : in std_logic;
        btn_south_in        : in std_logic;
        btn_west_in         : in std_logic;
        rot_centre_in       : in std_logic;
        rot_ccw             : in std_logic;
        rot_cw              : in std_logic;
        leds_out            : out std_logic_vector ( 7 downto 0 );
        switches_in         : in std_logic_vector ( 3 downto 0 ));
end entity user_logic ;


architecture IMP of user_logic is

    type State_Type is (READ, WRITE, AWAIT_READ, AWAIT_WRITE, INITIAL, STOP);

    signal buttons          : std_logic_vector(6 downto 0);
    signal memory_image     : std_logic_vector(255 downto 0);
    signal pos              : Natural range 0 to 7;
    signal state            : State_Type;
    signal mem_address      : std_logic_vector(31 downto 0);
    signal first_read       : std_logic;
begin

    buttons(0) <= btn_north_in;
    buttons(1) <= btn_east_in;
    buttons(2) <= btn_south_in;
    buttons(3) <= btn_west_in;
    buttons(4) <= rot_centre_in;
    buttons(5) <= rot_ccw;
    buttons(6) <= rot_cw;

    process (rst, clk) is
    begin
        if (rst = '1')
        then
            memory_image <= (others => '0');
            leds_out <= x"55";
            mem_address <= x"00000000";
            state <= INITIAL;
            first_read <= '1';
        elsif ((clk = '1')
        and (clk'event))
        then
            mem_read <= '0';
            mem_write <= '0';

            for i in 0 to 6
            loop
                if (buttons(i) = '1')
                then
                    memory_image ((i * 32) + 31 downto (i * 32)) 
                        <= memory_image ((i * 32) + 31 downto (i * 32)) + 1;
                end if;
            end loop;
            for i in 0 to 3
            loop
                memory_image (224 + (i * 4)) <= switches_in(i);
            end loop;
            case state is
            when INITIAL =>
                pos <= 0;
                mem_address <= x"8fffff00";
                state <= WRITE;
                -- leds_out <= x"20";
            when WRITE =>
                -- leds_out <= x"23";
                mem_data_out <= memory_image((pos * 32) + 31 downto (pos * 32));
                mem_write <= '1';
                state <= AWAIT_WRITE;
            when AWAIT_WRITE =>
                if (mem_busy = '0')
                then
                    if (pos = 7)
                    then
                        pos <= 0;
                        state <= READ;
                        -- leds_out <= x"24";
                    else
                        pos <= pos + 1;
                        state <= WRITE;
                        -- leds_out <= x"25";
                    end if;
                    mem_address <= mem_address + 4;
                end if;
            when READ =>
                mem_data_out <= (others => '0');
                if (first_read = '1')
                then
                    mem_write <= '1';
                else
                    mem_read <= '1';
                end if;
                state <= AWAIT_READ;
            when AWAIT_READ =>
                if (mem_busy = '0')
                then
                    leds_out <= mem_data_in (7 downto 0);
                    if ((mem_data_in (31 downto 24) = x"dd")
                    and (first_read = '0'))
                    then
                        state <= STOP;
                    else
                        state <= INITIAL;
                    end if;
                    first_read <= '0';
                end if;
            when STOP =>
                null;
            end case;
        end if;
    end process;

    mem_byte_enable <= "1111";
    mem_user_address <= mem_address;
    uart_data_out <= (others => '0');
    uart_read <= '0';
    uart_write <= '0';

end IMP;


